//
//	Copyright (C) 1997,1998,1999 Forest Edge Software, All Rights Reserved
//
//	Program:	eVAX, a "Virtual VAX" for Mac OS Computers
//
//	Author:		Tom Cole
//
//	Module:		console_dispatch.c
//
//	Purpose:	This routine accepts a single console command as an ASCIZ string,
//				decodes it, and runs the correct dispatcher.  It assumes that the
//				command has been correctly uppercased and contains only a single
//				command (no ';' allowed).
//
//	History:	07/28/97	New header format standardization
//
//

#include "vax.pch"
#include "console_proto.h"
#include "asmproto.h"

extern struct CONSOLE_DISPATCH_TABLE console_dispatch_table[];


//	Execute a console command

long console_dispatch( struct VAX ** Vax, char * cmd )
{
	struct VAX * vax;
	long rc;
	char *p;
	long verb;
	long n;
	console_handler routine;
	
	static char dispatcher_initialized = 0;

	if( !dispatcher_initialized ) {
		console_dispatch_init();
		dispatcher_initialized = 1;
	}
	
	
	vax = *Vax;
	p = cmd;
	rc = VAX_OK;

	if( vax && vax-> console.assembler_mode ) {
		rc = assemble( &vax, &p );
		return rc;
	}
	
	//	Read verb (must be unambigious in first 4 bytes
	
	read_verb( vax, &p, &verb );
	
	//	Find what command this is to dispatch
	
	routine = 0L;
	for( n = 0; console_dispatch_table[ n ].handler; n++ ) {
	
		if( console_dispatch_table[ n ].verb[ 0 ] == verb ||
		    console_dispatch_table[ n ].verb[ 1 ] == verb ||
		    console_dispatch_table[ n ].verb[ 2 ] == verb ) {
		    	routine = console_dispatch_table[ n ].handler;
		    	rc = (*routine)( Vax, &p );
		   		break;
		   	}
	}
	
	if( routine == 0L ) 
		rc = VAX_UNKCMD;

	if( rc == VAX_OK ) {
		flush_blanks( &p );
		if( !isend( vax, *p )) 
			rc = VAX_EXTRACMD;
	}
				
	return rc;
}

