//
//  Copyright (C) 1997,1998,1999 Forest Edge Software, All Rights Reserved
//
//  Program:    eVAX, a "Virtual VAX" for Mac OS Computers
//
//  Author:     Tom Cole
//
//  Module:     module.c
//
//  Purpose:    This console command displays memory storage in the virtual
//              VAX processor.
//
//  History:    07/28/97    New header format standardization
//
//              02/01/98    Modify the EXAM command to _only_ display the
//                          contents of memory.  Also, extend supported
//                          data types.  Finally, the final parameter is
//                          now an ending address, not a count.
//
//              01/06/99    Allow address(es) to be specified by expressions.
//                          Also fixed incorrect BIGENDIAN usages.

#include "vax.pch"
#include "console_proto.h"
#include "asmproto.h"
#include "fpu.h"
#include "pte.h"

long console_exam( struct VAX ** Vax, char ** P )
{
    long rc;
    struct VAX * vax;
    char * p, *saved_p;
    char * sname;
    
    long verb, size, count, maxbcount, bcount, len, maxlen;
    unsigned long value, n;
    
    long cp, d_long, d_long2;
    double dbl;
    
    short d_word;
    unsigned char d_byte;
    char ch, cbuff[ 64 ];
    
    union PTE pte;

/*
 *  Get local addressability to virtual VAX and command buffer
 */
 
    vax = *Vax;
    p = *P;

    rc = VAX_OK;
    
/*
 *  If there is no active virtual VAX, we are done!
 */
 
    if( vax == 0L ) {
        rc = VAX_NOVAX;
        return rc;
    }

/*
 *  Set the default length of the output.  If the type causes
 *  this to become zero, we don't allow or parse an end-point.
 */

    count = 0x7FFFFFFF;
    bcount = 0;
    maxbcount = 15;
    sname = 0L;
    size = 0;    

/*
 *  Look for size/format options that tell how to interpret the
 *  memory.
 */
    
    saved_p = p;
    rc = read_verb( vax, &p, &verb );
    
    if( rc == VAX_OK ) {
    
        switch( verb ) {
        
        case CHAR4('/','F','_','F'):
        case CHAR4('F','_','F','L'):
        
            verb = CHAR4('F','_','F','L');
            size = 4;
            break;
        
        case CHAR4('/','D','_','F'):
        case CHAR4('D','_','F','L'):
        
            verb = CHAR4('D','_','F','L');
            size = 8;
            break;
            
        case CHAR4('P','T','E',' '):
        
            size = 4;
            break;
            
        case CHAR4('D','E','S','C'):
        case CHAR4('D','S','C',' '):
        case CHAR4('/','D','S','C'):
        case CHAR4('/','D','E','S'):
        
            size = 1;
            count = -1;
            verb = CHAR4('D','E','S','C');
            break;
        
        case CHAR4('C','O','U','N'):
        case CHAR4('/','C','O','U'):
        
            size = 1;
            count = -1;
            verb = CHAR4('C','O','U','N');
            break;
        
        case CHAR4('N','U','L','L'):
        case CHAR4('Z','E','R','O'):
        case CHAR4('/','Z',' ',' '):
        case CHAR4('/','Z','E','R'):
            size = 1;
            count = -1;
            verb = CHAR4('Z','E','R','O');
            break;
            
        case CHAR4('A','S','C','I'):
        case CHAR4('/','A','S','C'):
        case CHAR4('/','A',' ',' '):
            size = 1;
            count = -1;
            maxbcount = 48; /* Max bytes per line; greater for plain ascii... */
            
            /* Get the ascii type.  If not a valid keyword, assume standard length */
            saved_p = p;
            read_verb( vax, &p, &verb );
            
            if( verb == CHAR4('Z','E','R','O') || verb == CHAR4('N','U','L','L') ||
                verb == CHAR4('D','E','S','C') || verb == CHAR4('D','S','C',' ') ||
                verb == CHAR4('C','O','U','N') ) {
                size = 1;
                count = -1;
            }
            else {
                verb = CHAR4('A','S','C','I');
                p = saved_p;
                count = 16;
            }
                
            break;
            
        case CHAR4('B','Y','T','E'):
        case CHAR4('/','B','Y','T'):
        case CHAR4('/','B',' ',' '):
            size = 1;
            verb = CHAR4('B','Y','T','E');
            break;
        
        case CHAR4('W','O','R','D'):
        case CHAR4('/','W','O','R'):
        case CHAR4('/','W',' ',' '):
            size = 2;
            verb = CHAR4('W','O','R','D');
            break;
        
        case CHAR4('L','O','N','G'):
        case CHAR4('/','L','O','N'):
        case CHAR4('/','L',' ',' '):
            size = 4;
            verb = CHAR4('L','O','N','G');
            break;

        /* In the default case, assume dumping longs, and back up the token pointer */
        
        default:
            verb = CHAR4('L','O','N','G');
            size = 4;
            p = saved_p;
        }
    }
    
    
    
    /*  Get starting address */
    
    rc = asm_expr( vax, &p, ( long * ) &value );
    if( rc )
        goto exit;

    
    /*  Get ending location, if the type hasn't disallowed it */
        
    if( count > 0 ) {
        flush_blanks( &p );
        if( !isend( vax, *p )) {
            rc = asm_expr( vax, &p, ( long *) &count );
            if( rc )
                goto exit;
            count = count - value;
            if( count < 1 ) {
                rc = VAX_INVENDADDR;
                goto exit;
            }
        }
    }

/*
 *  If the ending count was never given it's still MAXINT.  If so, then let's
 *  assume we go for one iteration only.
 */

    if( count == 0x7FFFFFFF )
        count = 0;
        
/*
 *  Handle the special cases now.  These are cases where we format
 *  memory as datastructures.  The verb contains the type of the
 *  data structure.  If it's an unknown one, then bail out...
 */
 
    if( count < 0 ) {

        switch( verb ) {
        
        /*  Determine the size as a counted string */
        
        case CHAR4('C','O','U','N'):
        case CHAR4('/','C','O','U'):
        
            rc = load_memory( vax, value, ( void * ) &d_word, 2 );
			if( rc )
                return rc;
            value = value + 2;
            count = d_word;
            verb = CHAR4('A','S','C','I');
            maxbcount = 48;
            sname = "counted ASCII";
            break;
            
            
    
        /*  Determine the size and length from a descriptor */
        
        case CHAR4('D','E','S','C'):
        case CHAR4('D','S','C',' '):
        case CHAR4('/','D','E','S'):
        case CHAR4('/','D','S','C'):
        
            rc = load_memory( vax, value, ( void * ) &d_word, 2 );
			if( rc )
                return rc;
            rc = load_memory( vax, value + 4, ( void * ) &d_long, 4 );
			if( rc )
                return rc;
                
            verb = CHAR4('A','S','C','I');
            maxbcount = 48;
            sname = "descriptor";
            
            value = d_long;
            count = d_word;
            break;
        
        case CHAR4('N','U','L','L'):
        case CHAR4('Z','E','R','O'):
        case CHAR4('/','Z','E','R'):
        case CHAR4('/','Z',' ',' '):
        
            for( count = 0; count < 1024; count++ ) {
                rc = load_memory( vax, value + count, ( void * ) &ch, 1 );
				if( rc )
                    return rc;
                    
                if( ch == 0 )
                    break;
            }
            verb = CHAR4('A','S','C','I');
            maxbcount = 48;
            sname = "null-terminated ASCII";
            
            break;
            
        default:
            rc = VAX_UNKSTRUCT;
            goto exit;
        }
            
    }
    
    
    len = 0;
    cp = 0;
    cbuff[ 0 ] = 0;
    maxlen = 0;
    bcount = 0;


    if( sname != 0L )
        printf( "Length of %s string is %ld (%04lX) bytes\n", sname, count, count );
        
    for( n = value; n <= value + count; n += size ) {

        if( len == 0 ) {
            printf( "%08lX: ", n );
            cp = 0;
        }
            
        switch( verb ) {

        case CHAR4('F','_','F','L'):
        
            rc = load_memory( vax, n, ( void * ) &d_long, 4 );
			if( rc )
                return rc;
            
            bcount += 4;
            
            rc = fpu_load( vax, d_long, 0L, &dbl );
            if( rc )
                return rc;
            
            printf( "%9.9f\n", dbl );
            break;
        
        case CHAR4('D','_','F','L'):
        
            rc = load_memory( vax, n, ( void * ) &d_long, 4 );
			if( rc )
                return rc;
            
            rc = load_memory( vax, n + 4, ( void * ) &d_long2, 4 );
			if( rc )
				return rc;
            bcount += 8;
            
            rc = fpu_load( vax, d_long, d_long2, &dbl );
            if( rc )
                return rc;
            
            printf( "%12.9f\n", dbl );
            break;
            
            
        case CHAR4('P','T','E',' '):
        
            rc = load_memory( vax, n, ( void * ) &pte.longword, 4 );
			if( rc )
                return rc;
                
            bcount += 4;
            
            printf( "%08lX   V:%d  PROT: %03d   M: %d  OWN: %02d  PFN: %08lX  S:%02d\n",
                    pte.longword,
                    pte.bit.v ? 1 : 0,
                    ( unsigned int ) pte.bit.prot,
                    pte.bit.m ? 1 : 0,
                    ( unsigned int ) pte.bit.own,
                    ( long ) pte.bit.pfn << 9,  /* Recalc as address */
                    ( unsigned int ) pte.bit.s );
                    
            break;
            
        case CHAR4('A','S','C','I'):
        
            rc = load_memory( vax, n, ( void * ) &ch, 1 );
			if( rc )
                return rc;
                
            if( ch < ' ' || ch > 127 )
                ch = '.';
                
            printf( "%c",ch );
            len += 1;
            bcount += 1;
            break;
            
        case CHAR4('B','Y','T','E'):
            
            rc = load_memory( vax, n, ( void * ) &d_byte, 1 );
			if( rc )
                return rc;
            
            printf( "%02lX ", (long ) d_byte );
            cbuff[ cp++ ] = d_byte;
            len += 3;
            bcount += 1;
            break;
                                
        case CHAR4('W','O','R','D'):
        
            rc = load_memory( vax, n, ( void * ) &d_word, 2 );
			if( rc )
                return rc;

#if 1 /* !BIGENDIAN */

            printf( "%02lX%02lX ", 
                ( unsigned long ) (( d_word & 0xFF00 ) >> 8 ),
                ( unsigned long ) (( d_word & 0x00FF ) >> 0 ));
            
            /* Note reverse order from hex display */
            cbuff[ cp++ ] = ( char ) (( d_word & 0x00FF ) >> 0 );
            cbuff[ cp++ ] = ( char ) (( d_word & 0xFF00 ) >> 8 );

#else
            printf( "%02lX%02lX ", 
                ( unsigned long ) (( d_word & 0x00FF ) >> 0 ),
                ( unsigned long ) (( d_word & 0xFF00 ) >> 8 ));
            
            /* Note reverse order from hex display */
            cbuff[ cp++ ] = (( d_word & 0xFF00 ) >> 8 );
            cbuff[ cp++ ] = (( d_word & 0x00FF ) >> 0 );

#endif

            bcount += 2;
            len += 5;
            break;
            
        case CHAR4('L','O','N','G'):
        
            rc = load_memory( vax, n, ( void * ) &d_long, 4 );
			if( rc )
                return rc;


#if 1 /*  !BIGENDIAN */

            printf( "%02lX%02lX%02lX%02lX ", 
                ( unsigned long ) (( d_long & 0xFF000000L ) >> 24 ),
                ( unsigned long ) (( d_long & 0x00FF0000L ) >> 16 ),
                ( unsigned long ) (( d_long & 0x0000FF00L ) >>  8 ),
                ( unsigned long ) (( d_long & 0x000000FFL ) >>  0 ));

            /* Note reverse order from hex display */                           
            cbuff[ cp++ ] = ( unsigned char ) (( d_long & 0x000000FFL ) >>  0 );
            cbuff[ cp++ ] = ( unsigned char ) (( d_long & 0x0000FF00L ) >>  8 ),
            cbuff[ cp++ ] = ( unsigned char ) (( d_long & 0x00FF0000L ) >> 16 ),
            cbuff[ cp++ ] = ( unsigned char ) (( d_long & 0xFF000000L ) >> 24 ),
            

#else
            printf( "%02lX%02lX%02lX%02lX ", 
                ( unsigned long ) (( d_long & 0x000000FFL ) >>  0 ),
                ( unsigned long ) (( d_long & 0x0000FF00L ) >>  8 ),
                ( unsigned long ) (( d_long & 0x00FF0000L ) >> 16 ),
                ( unsigned long ) (( d_long & 0xFF000000L ) >> 24 ));

            /* Note reverse order from hex display */                           
            cbuff[ cp++ ] = ( unsigned char ) (( d_long & 0xFF000000L ) >> 24 ),
            cbuff[ cp++ ] = ( unsigned char ) (( d_long & 0x00FF0000L ) >> 16 ),
            cbuff[ cp++ ] = ( unsigned char ) (( d_long & 0x0000FF00L ) >>  8 ),
            cbuff[ cp++ ] = ( unsigned char ) (( d_long & 0x000000FFL ) >>  0 );
            
#endif

            bcount += 4;
            len += 9;
            break;
        }
        
    /*
     *  See if we've hit the end-of line and need to dump the character data.
     */
     
        if( bcount > maxbcount ) {
        
            maxlen = len;
            for( len = 0; len < cp; len++ ) {
                ch = cbuff[ len ];
                if( ch < ' ' || ch > 127 )
                    cbuff[ len ] = '.';
            }
            cbuff[ cp ] = 0;
            cp = 0;
            len = 0;
            bcount = 0;
            printf( "   %s\n", cbuff );
        }
    }

    if( len > 0 ) {
    
        for( ; len < maxlen; len++ )
            printf( " " );
            
        for( len = 0; len < cp; len++ ) {
            ch = cbuff[ len ];
            if( ch < ' ' || ch > 127 )
                cbuff[ len ] = '.';
        }
        cbuff[ cp ] = 0;
        cp = 0;
        len = 0;
        printf( "   %s\n", cbuff );
    }
    
    printf( "\n" );
            
    
/*
 *  Store parameters back to caller.
 */

exit:

    *Vax = vax;
    *P = p;
    return rc;
}
