
//
//  Copyright (C) 1997,1998,1999 Forest Edge Software, All Rights Reserved
//
//  Program:    eVAX, a "Virtual VAX" for Mac OS Computers
//
//  Author:     Tom Cole
//
//  Module:     step.c
//
//  Purpose:    This module implements the STEP command, which single steps
//				the processor starting at a specific address, or the current
//				PC if no address is given.
//
//  History:    06/03/99    New header format standardization
//
//              06/03/99	Added ability to show what registers changed on
//							a step boundary.  This only works for STEP command,
//							not disassembly mode.  On by default, this feature
//							is controlled by SET DEBUG [NO]REGISTERS
//


#include "vax.pch"
#include "console_proto.h"
#include "asmproto.h"

long console_step( struct VAX ** Vax, char ** P )
{
	long rc;
	struct VAX * vax;
	char * p;
	unsigned long value;
	unsigned long n;
	unsigned long psl, reg[ 16 ];
	static char * regname[ 16 ] = 
	{ "R0", "R1", "R2", "R3", "R4", "R5", "R6", "R7", "R8", "R9", "R10", "R11",
	"AP", "FP", "SP", "PC" };
	static char * blanks = "                    ";

/*
 *	Get local addressability to virtual VAX.  
 */
 
	vax = *Vax;
	p = *P;
	psl = 0L;

/*
 *	If there is no active virtual VAX, we are done!
 */
 
	if( vax == 0L ) {
		rc = VAX_NOVAX;
		return rc;
	}

/*
 *	We basically do a normal execute but with the special console single step flag
 */
 
	vax-> console.singlestep = 1;

	flush_blanks( &p );
	
	if( !isend( vax, *p )) {
		rc = asm_hex( vax, &p, &value );
		if( rc )
			return rc;;
		vax-> PC = value;
	}

/*
 *	Capture the register state before the step.
 */

	if( vax-> debug & DBG_REGISTERS ) {

		for( n = 0; n < 14; n++ )
			reg[ n ] = vax-> reg[ n ];
	
		write_psl_bits( vax );
		psl = vax-> psl.reg;
	}

/*
 *	Note that in SST mode, we always set the disassemble flag.
 */
 
	rc = execute_vax( vax, 1 );

/*
 *	What registers changed?
 */

	if( vax-> debug & DBG_REGISTERS ) {

		for( n = 0; n < 14; n++ ) {
			if( reg[ n ] != vax-> reg[ n ] ) {
				if( n < 12 )
					printf( "%s%3s:  %08lX  %ld\n", blanks, regname[ n ], vax-> reg[ n ], vax-> reg[ n ]);
				else
					printf( "%s%3s:  %08lX\n", blanks, regname[ n ], vax-> reg[ n ] );
			}
		}
		write_psl_bits( vax );
		if( psl != vax-> psl.reg )
			printf("%sPSL:  %08lX\n", blanks, psl );
	}


	
/*
 *	Store parameters back to caller.
 */
 

	*Vax = vax;
	*P = p;
	return rc;
}

