//
//  Copyright (C) 1997,1998,1999 Forest Edge Software, All Rights Reserved
//
//  Program:    eVAX, a "Virtual VAX" for Mac OS Computers
//
//  Author:     Tom Cole
//
//  Module:     emul_ash.c
//
//  Purpose:    Emulator handlers for ROTL, ASHL and ASHQ instructions
//
//
//  History:    07/28/97    New header format standardization
//
//


#include "vax.pch"

#define LEFT 'L'
#define RIGHT 'R'


//  The ROTL instruction

EMULATOR_ENTRY( emul_rotl )
{

    long direction, count, n;
    unsigned long value, bit;
    long *dest_a;

//  Get the count of bit(s) to move the value

	GET_OPERAND( count, long, 0, OP_RD );


    if( count < 0 ) {
        direction = RIGHT;
        count = -count;
    }
    else
        direction = LEFT;

//  Get the source longword value.

	GET_OPERAND( value, unsigned long, 1, OP_RD );
    
//  Verify that the destination is accessible

    dest_a = ( long * ) get_operand( vax, opcode, 2, OP_WR );
    if( dest_a == 0L )
        return VAX_FAULT;


//  Based on direction, shift the right number of times.

    if( direction == RIGHT ) {
    
        for( n = 0; n < count; n++ ) {
            bit = value & 0x00000001;
            value = value >> 1;
            if( bit )
                value = value + 0x80000000UL;
        }
    }
    else {
    
        for( n = 0; n < count; n++ ) {
        
            bit = value & 0x80000000UL;
            value = value << 1;
            if( bit )
                value = value + 0x00000001UL;
        }
    }

    SETCONDITIONBITS( value, 0L );
    vax-> pslw.v = 0;
    
    return put_operand( vax, opcode, 2, OP_WR, ( void * ) &value );
    
}


//  The ASHL and ASHQ instruction(s)

EMULATOR_ENTRY( emul_ash )
{


    long source, *source_a;
    signed char count, *count_a;
    long data;
    
    //  Get address of count operand byte
    
    count_a = ( signed char * ) get_operand( vax, opcode, 0, OP_RD );
    if( count_a == 0L )
        return VAX_FAULT;
    count = *count_a;
    
    //  Get address of data to shift.
    
    source_a = ( long * ) get_operand( vax, opcode, 1, OP_RD );
    if( source_a == 0L )
        return VAX_FAULT;
    

    switch( opcode-> function ) {
    
    case 0x78:  /* ASHL */
    
        source = *source_a;
        
        if( count < 0 ) {
        
            if( count <= -31 )
                data = ( source < 0 ) ? -1 : 0;
            else
                data = sext(( source >> count ), 4 );
        }
        else {
        
            if( count > 32 )
                data = 0L;
            else
                data = source << count;
        }
        
        SETCONDITIONBITS( data, 0L );
    
        put_operand( vax, opcode, 2, OP_WR, ( void * ) &data );
        break;
        
    default:
    
        set_fault( vax, EXC_PRIV, 0 );
        return VAX_FAULT;
    
    }
    


    return VAX_OK;
}


