//
//  Copyright (C) 1997,1998,1999 Forest Edge Software, All Rights Reserved
//
//  Program:    eVAX, a "Virtual VAX" for Mac OS Computers
//
//  Author:     Tom Cole
//
//  Module:     emul_xfc.c
//
//  Purpose:    Emulator handlers for XFC instructions which provide exits to
//              operating system like functions.
//
//  History:    07/28/97    New header format standardization
//              08/16/98    Added XFC XFC$VM to test VM translations
//
//


#include "vax.pch"
#include "shim.h"

long console_dispatch( struct VAX ** Vax, char * cmd );




EMULATOR_ENTRY( emul_xfc )
{
    char ch;
    short loop;
    short slen;
    long saved_mapen;
    long code;
    long addr, rc;
    char * buff;
    
//  The XFC instruction takes a single implicit immediate instruction argument.  Get it.

    code = opcode-> VAXaddr[ 0 ];

//  Based on the code, do some work.


    switch( code ) {
    
    case    0x01:   /* #XFC$CONSOLE_WRITE       R0 = byte to write */
    
            ch = ( char ) ( vax-> R0 & 0x000000FFL );
            putchar( ch );
            break;
    
    case    0x02:   /*  #XFC$CONSOLE_READ       R0 = byte read */
    
            ch = ( char ) getchar();
            vax-> R0 &= 0xFFFFFF00L;
            vax-> R0 |= ch;
            break;
    
    case    0x03:   /*  #XFC$CONSOLE_CMD        R0 = address of command text */
    
            /* Get addressability to data, and fetch the length value */
            
            addr = vax-> R0;
            load_memory( vax, addr, ( void * ) &slen, 2 );
            addr += 2;
            
            /* If the string is empty or badly formed, don't use it. */
            
            if( slen <= 0 )
                break;
            
            /* Allocate a buffer and copy the string to the buffer */
            
            buff = ( char * ) getmem( slen + 2 );
            
            for( loop = 0; loop < slen; loop++ ) {
                load_memory( vax, addr + loop, ( void * ) &ch, 1 );
                buff[ loop ] = ch;
            }
            
            /* Slap a null terminator and execute as a console command */
            
            buff[ loop ] = '\0';
            
            rc = console_dispatch( &vax, buff );
            
            /* Store the return code back to the program */
            
            vax-> R0 = rc;
            
            
            /* Free the buffer now that we're done with it. */
            
            freemem( buff );

            break;
    
    case	0x7D:	/*	#XFC$SHIM		R0 = function dispatch code */
    
    		return shim( vax );
    		
    case    0x7F:   /*  #XFC$VMR        R0 = address to translate   */
                    /*                  R0 receives updated address */
                    /*                  V  bit set if TNV fault     */
            
    case    0x7E:   /*  #XFC$VMW        R0 = address to translate   */
                    /*                  R0 receives updated address */
                    /*                  V  bit set if TNV fault     */
            
            addr = vax-> R0;
            slen = 4;
            
            saved_mapen = vax-> MAPEN;
            vax-> MAPEN = 1;
            if( vm( vax, ( unsigned long * ) &addr, &slen, ( short ) ( code == 0x7E )) 
                == VAX_FAULT )
                         vax-> pslw.v = 1;
            else
                vax-> R0 = addr;
            vax-> MAPEN = saved_mapen;
            
            
            break;

            
    default:
            set_fault( vax, EXC_RESOP, 0 );
    }

    return VAX_OK;
    
}
